#ifndef _CHRONOPTICS_TOF_LOG_H_
#define _CHRONOPTICS_TOF_LOG_H_

#include <chronoptics/tof/error.h>

#ifdef __cplusplus
extern "C" {
#endif

/** Function pointer type for log callback
 * @param log_msg String containing log messages
 * @param user_data The passed back user data
 */
typedef void (*tof_log_callback_t)(const char* log_msg, void* user_data);

/** Function pointer type for log callback separated
 * @param level Log level. Info 2, Warning 3, Error 4, Critical 5
 * @param logger_name The name of the logger
 * @param msg Actual message
 * @param user_data The passed back user data
 */
typedef void (*tof_log_callback_separated_t)(int16_t level, const char* logger_name, const char* msg, void* user_data);

/** Class containing log information about the crash that occurred
 */
typedef struct tof_crash_report* tof_crash_report_t;

/** Destruct tof_crash_report */
TOF_EXPORT void tof_crash_report_delete(tof_crash_report_t ptr);

/** Generate crash report on a camera
 * @param ptr Pointer to class
 * @param serial Camera to generate the crash report on
 * @param description Description of what happened
 * @param error Pointer to error
 * @return New object
 */
TOF_EXPORT tof_crash_report_t tof_crash_report_new(const char* serial, const char* description, tof_error_t *error);

/** Get the description
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return Description
 */
TOF_EXPORT const char* tof_crash_report_get_description(const tof_crash_report_t ptr, tof_error_t *error);

/** Set the description
 * @param ptr Pointer to class
 * @param description Description
 * @param error Pointer to error
 */
TOF_EXPORT void tof_crash_report_set_description(tof_crash_report_t ptr, const char* description, tof_error_t *error);

/** Get the complete report
 * @param ptr Pointer to class
 * @param error Pointer to error
 * @return String of the entire report
 */
TOF_EXPORT const char* tof_crash_report_get_report(const tof_crash_report_t ptr, tof_error_t *error);

/** Save report to disk
 * @param ptr Pointer to class
 * @param file_name File to save the report to
 * @param error Pointer to error
 */
TOF_EXPORT void tof_crash_report_save(const tof_crash_report_t ptr, const char* file_name, tof_error_t *error);

/** Get the version of the tof library
 * @param error Pointer to error
 * @return Library version
 */
TOF_EXPORT const char* tof_get_version(tof_error_t *error);

/** Log info/warning/error occuring in the tof library to stdout
 * @param error Pointer to error
 */
TOF_EXPORT void tof_log_to_console(tof_error_t *error);

/** Log info/warning/error occuring in the tof library
 * @param file_location Location where to store the log file
 * @param error Pointer to error
 */
TOF_EXPORT void tof_log_to_file(const char* file_location, tof_error_t *error);

/** Log info/warning/error occuring in the tof library to function
 * @param callback Function that is called with log data
 * @param callback_user_data User data that will be passed back when the function pointer is called
 * @param error Pointer to error
 */
TOF_EXPORT void tof_log_callback(tof_log_callback_t callback, void* callback_user_data, tof_error_t *error);

/** Log info/warning/error occuring in the tof library to function
 * @param callback_separated Function that is called with separated log data
 * @param callback_separated_user_data User data that will be passed back when the function pointer is called
 * @param error Pointer to error
 */
TOF_EXPORT void tof_log_callback_separated(tof_log_callback_separated_t callback_separated, void* callback_separated_user_data, tof_error_t *error);

/** Stop logging
 * @param error Pointer to error
 */
TOF_EXPORT void tof_log_drop(tof_error_t *error);

#ifdef __cplusplus
}
#endif

#endif
